;; Remote Control For Cyborg Chicken Fight
;;
;; Copyright 2017 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike@mikekohn.net
;;
;; Remote control circuit used for the Cyborg Chicken Fight

.msp430
.include "msp430x2xx.inc"

RAM equ 0x0200
DATA equ RAM+32
STOP_COUNT equ RAM
DEBUG equ RAM+2
WAIT_SEQUENCE_START equ RAM+2
WAIT_SEQUENCE_END equ RAM+4

;  r4 = data pointer
;  r5 = sent bit count
;  r6 = joystick input
;  r7 = next_command
;  r8 = controller bits
;  r9 = interrupt count
; r10 = 0=keep IR off, 1=let IR blink
; r11 = interrupt count peak (bit length)
; r12 = wait_sequence
; r13 = wait_count
; r14 =
; r15 = temp in interrupt

; Command byte is
; SScccCCC
; C = 0: stop moving
; C = 1: forward
; C = 2: down
; C = 3: left
; C = 4; right
; C = 5; fire
; ccc is a repeat of CCC
; SS = 01: controller 1
; SS = 10: controller 2

; 16,000,000 / (38000 * 2) = 210.52
; 76.0kHz = 13.1 microseconds
;
; header = 2ms / 2ms  = 152 interrupts on / 152 off
; short  = 0.3ms = 23 interrupts
; long   = 0.6ms = 46 interrupts

HEADER_ON equ 152
HEADER_OFF equ 152
SHORT equ 23
LONG equ 46
GAP_LENGTH equ 12000

.org 0xf800
start:
  ;; Turn off watchdog
  ;mov.w #(WDTPW|WDTHOLD), &WDTCTL

  ;; Set watchdog to clock / 32768.
  mov.w #(WDTPW|WDTIS0), &WDTCTL

  ;; Turn off interrupts.
  dint

  ;; Set up stack pointer
  mov.w #0x0280, SP

  ;; Set MCLK to 16 MHz with DCO
  mov.b #DCO_4, &DCOCTL
  mov.b #RSEL_15, &BCSCTL1
  mov.b #0, &BCSCTL2

  ;; Set up output pins
  ;; P1.0 = IR LED
  ;; P1.1 = DEBUG LED
  ;; P1.3 = Joystick Fire
  ;; P1.4 = Joystick Up
  ;; P1.5 = Joystick Down
  ;; P1.6 = Joystick Left
  ;; P1.7 = Joystick Right
  ;; P2.6 = Controller Select
  mov.b #0x07, &P1DIR
  mov.b #0xf8, &P1OUT
  mov.b #0xf8, &P1REN
  mov.b #0x00, &P2DIR
  mov.b #0x40, &P2OUT
  mov.b #0x40, &P2REN
  mov.b #0x00, &P2SEL
  mov.b #0x00, &P2SEL2

  ;; Set up Timer
  mov.w #211, &TACCR0  ; 16,000,000 / (38000 * 2) = 210.52
  mov.w #(TASSEL_2|MC_1), &TACTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TACCTL0
  mov.w #0, &TACCTL1

  ;; Small delay
.if 0
  mov.w #0, r15
delay:
  dec.w r15
  jnz delay
.endif

  ;; Change controller number parameters based on jumper.
  bit.b #0x40, &P2IN
  jnz set_controller_1
  mov.b #0x40, r8
  mov.w #wait_sequence_start_0, &WAIT_SEQUENCE_START
  mov.w #wait_sequence_end_0, &WAIT_SEQUENCE_END
  jmp set_controller_done
set_controller_1:
  mov.b #0x80, r8
  mov.w #wait_sequence_start_1, &WAIT_SEQUENCE_START
  mov.w #wait_sequence_end_1, &WAIT_SEQUENCE_END
set_controller_done:

  mov #0, r10
  mov #1, r13
  mov &WAIT_SEQUENCE_START, r12
  mov.w #0, &STOP_COUNT

  ;; Enable interrupts.
  eint

main:
  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL

  mov.b &P1IN, r6
  bit.b #0x08, r6
  jnz main_not_fire
  mov.b #5, r6
  jmp main_joystick_done
main_not_fire:
  bit.b #0x10, r6
  jnz main_not_up
  mov.b #1, r6
  jmp main_joystick_done
main_not_up:
  bit.b #0x20, r6
  jnz main_not_down
  mov.b #2, r6
  jmp main_joystick_done
main_not_down:
  bit.b #0x40, r6
  jnz main_not_left
  mov.b #3, r6
  jmp main_joystick_done
main_not_left:
  bit.b #0x80, r6
  jnz main_not_right
  mov.b #4, r6
  jmp main_joystick_done
main_not_right:
  mov.b #0, r6
main_joystick_done:

  ;; Check if current command is 0
  mov.b r6, r7
  cmp #0, r7
  jz main_dont_update_stop_count
  ;; If the joystick is being pushed, set STOP_COUNT to 5 so that
  ;; when the joystick is let go, it will send 5 (or 4) stop commands before
  ;; going silent.
  mov.w #5, &STOP_COUNT
main_dont_update_stop_count:

  ;; Decide if a command should be sent
  ;; if STOP_COUNT == 0 then do not send anything
  cmp.w #0, &STOP_COUNT
  jeq main
  ;; if wait_count != 0 then do not send anything
  cmp.w #0, r13
  jne main

  sub.w #1, &STOP_COUNT

  mov.w @r12, r13
  add.w #2, r12
  cmp.w &WAIT_SEQUENCE_END, r12
  jne main_dont_reset_seq
  mov.w &WAIT_SEQUENCE_START, r12
main_dont_reset_seq:

  mov.b r7, r6
  rla.b r6
  rla.b r6
  rla.b r6
  bis.b r7, r6
  bis.b r8, r6  ; controller number

  call #send_command
  jmp main

send_command:
  ;; Debug LED on
  bis.b #2, &P1OUT 

  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL

  ;mov.b r6, &DEBUG
  ;mov.b r7, &DEBUG+1

  ;; Send header on.
  clr.w r9
  mov.b #1, r10
wait_header_on:
  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL
  cmp.w #HEADER_ON, r9
  jne wait_header_on
  mov.b #0, r10
  bic.b #1, &P1OUT

  ;; Send header off.
  clr.w r9
wait_header_off:
  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL
  cmp.w #HEADER_OFF, r9
  jne wait_header_off

  ;; for (r5 = 0; r5 < 9; r5++)
  mov.w #9, r5
next_bit:

  ;; Kick the dog
  mov.w #(WDTPW|WDTCNTCL), &WDTCTL

  ;; Turn bit on.
  clr.w r9
  mov.b #1, r10
wait_bit_on:
  cmp.w #SHORT, r9
  jne wait_bit_on
  mov.b #0, r10
  bic.b #1, &P1OUT

  ;; Compute length of bit.
  mov.w #SHORT, r11
  bit.w #0x80, r6
  jz is_short
  mov.w #LONG, r11
is_short:

  ;; Turn bit off
  clr.w r9
wait_bit_off:
  cmp.w r11, r9
  jne wait_bit_off

  ;; Shift bit and increment count and possibly command pointer.
  rla.b r6
  dec.w r5
  jnz next_bit

  ;; Turn bit on. (stop bit)
  clr.w r9
  mov.b #1, r10
wait_stop_bit:
  cmp.w #SHORT, r9
  jne wait_stop_bit
  mov.b #0, r10
  bic.b #1, &P1OUT

  ;; Debug LED off
  bic.b #2, &P1OUT

  ;; Done.
  ret

timer_interrupt:
  xor.b r10, &P1OUT
  inc.w r9

  ;; if wait_count == 0 then exit interrupt
  cmp.w #0, r13
  jeq timer_interrupt_exit

  ;; wait_count--
  dec.w r13

timer_interrupt_exit:
  reti

osc_fault:
  bic. b #OFIFG, &IFG1
  jmp main

wait_sequence_start_0:
  dw 7600, 5320, 11400, 9000 
wait_sequence_end_0:
wait_sequence_start_1:
  dw 5320, 7600, 9000, 11400
wait_sequence_end_1:

  ;; Vectors
.org 0xfff2
  dw timer_interrupt
;.org 0xfff4
;  dw watch_dog
.org 0xfffc
  dw osc_fault
.org 0xfffe
  dw start                 ; Reset


